/*
  GRANITE DATA SERVICES
  Copyright (C) 2011 GRANITE DATA SERVICES S.A.S.

  This file is part of Granite Data Services.

  Granite Data Services is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as published by
  the Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.

  Granite Data Services is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public License
  for more details.

  You should have received a copy of the GNU Library General Public License
  along with this library; if not, see <http://www.gnu.org/licenses/>.
*/

package org.granite.osgi;

public interface GraniteClassRegistry {

    /**
     * <p>Register classes to a destination.</p>
     * <p>Note: If <code>destination</code> is already registered,then <code>classes</code> are appended to the already
     * registered classes list.</p>
     *
     * @param destination The granite destination on which classes are registered
     * @param classes     A set of classes to register
     */
    public void registerClasses(String destination, Class[] classes);

    /**
     * <p>Unregister classes from a given granite destination</p>
     * <p>If <code>classes</code> are not provided then the destination is removed. The destination is also removed if
     * there is no more classes associated to <code>destination</code> after removing the given <code>classes</code>.</p>
     *
     * @param destination The granite destination on which classes are registered
     * @param classes     An optional set of classes to unregister
     */
    public void unregisterClasses(String destination, Class... classes);
}
