/*
 GRANITE DATA SERVICES
 Copyright (C) 2011 GRANITE DATA SERVICES S.A.S.

 This file is part of Granite Data Services.

 Granite Data Services is free software; you can redistribute it and/or modify
 it under the terms of the GNU Library General Public License as published by
 the Free Software Foundation; either version 2 of the License, or (at your
 option) any later version.

 Granite Data Services is distributed in the hope that it will be useful, but
 WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 FITNESS FOR A PARTICULAR PURPOSE. See the GNU Library General Public License
 for more details.

 You should have received a copy of the GNU Library General Public License
 along with this library; if not, see <http://www.gnu.org/licenses/>.
 */

package org.granite.util {

import flash.net.registerClassAlias;
import flash.utils.Dictionary;

import mx.collections.IList;

public class GraniteClassRegistry {

    private static var destinationClasses:Dictionary = new Dictionary();

    /**
     * <p>Register classes to a destination.</p>
     * <p>Note: If <code>destination</code> is already registered, <code>aliasMap</code> is appended to the registered classes.</p>
     * @param destination The granite destination on which classes are registered
     * @param aliasMap A dictionary of Flex classes and their aliases to register
     */
    public static function registerClasses(destination:String, aliasMap:Dictionary):void {
        var currentAliasMap:Dictionary = destinationClasses[destination];
        if (currentAliasMap) {
            for (var alias:String in aliasMap) {
                currentAliasMap[alias] = aliasMap[alias];
            }
        } else {
            currentAliasMap = aliasMap;
        }
        destinationClasses[destination] = currentAliasMap;
    }

    /**
     * <p>Unregister classes from a given granite destination</p>
     * <p>If <code>aliases</code> is not provided or null then the destination is removed</p>
     * @param destination The granite destination on which classes are registered
     * @param aliases An optional List of aliases of Flex classes to unregister
     */
    public static function unregisterClasses(destination:String, aliases:IList = null):void {
        if (aliases != null) {
            var currentAliasMap:Dictionary = destinationClasses[destination];
            if (currentAliasMap) {
                for each (var alias:String in aliases) {
                    delete currentAliasMap[alias];
                }
                if (getLength(currentAliasMap) != 0) {
                    destinationClasses[destination] = currentAliasMap;
                } else {
                    delete destinationClasses[destination];
                }
            }
        } else {
            delete destinationClasses[destination];
        }
    }


    public static function useClasses(destination:String):void {
        var aliasMap:Dictionary = destinationClasses[destination];
        if (aliasMap) {
            for (var alias:String in aliasMap) {
                registerClassAlias(alias, aliasMap[alias]);
            }
        }
    }

    private static function getLength(o:Object):uint {
        var len:uint = 0;
        for (var item:* in o)
            if (item != "mx_internal_uid")
                len++;
        return len;
    }
}
}